#!/usr/perl5/bin/perl -w
#
# Copyright (c) 2001 by Sun Microsystems, Inc.
# All rights reserved.
#
#ident	"@(#)nfs.pl	1.14	01/08/23 SMI"
#

use lib '/opt/SUNWscvw/lib/perl';
use Socket;
use Cluster::common;
use Cluster::Common;
use Cluster::scrgcommon;
use strict;
use Sun::Solaris::Utils qw(gettext);

&do_page("Configure NFS"); # gettext("Configure NFS")
exit;

sub cmd_mainpage {
    my $i;
    &wizard_start;
    print $q->_script($q->yoke_jscript("c"));
    &getscrgconfig();
    if (! &checktype("SUNW.nfs")) {
	    return;
    }

    if (! (defined $FORM{"create"})) {
	foreach $i (keys %rsrt) {
	    if ($rsrt{$i} eq "SUNW.nfs") {
		# Bail out of this page and print decision page
		&decide();
		&wizard_end();
		return;
	    }
	}
    }


    print "<h2>".gettext("This page will create a HA-NFS resource.")."</h2>\n";

    if (!&getlh()) {
	return;
    }

    &action_table_start();

    if ($errpos == 2) {
	&errmsg();
    }
    if ($FORM{"exportdir"} eq "") {
	$FORM{"exportdir"} = &getglobal();
    }
    &action_table_row_start(gettext("Global Directory to Export:"));
    &input("exportdir", 35);
    &action_table_row_end();

    if ($errpos == 3) {
	&errmsg();
    }
    if ($FORM{"admindir"} eq "") {
	&getrgprops($lhrg[0]);
	# Use existing Pathprefix for the logical hostname, if any
	if ($rgprops{"Pathprefix"}) {
	    $FORM{"admindir"} = $rgprops{"Pathprefix"};
	} else {
	    $FORM{"admindir"} = &getglobal;
	}
    }
    if (! -d $FORM{"admindir"} || ! &isglobal($FORM{"admindir"})) {
	# Don't use faulty default
	$FORM{"admindir"} = "";
    }
    &action_table_row_start(gettext("Global Directory to Hold the NFS Administration Directory:"));
    &input("admindir", 35);
    &action_table_supplement(gettext("(i.e. the Pathprefix)"));
    &action_table_row_end();

    if ($errpos == 101) {
	&errmsg();
    }
    &action_table_row_start(gettext("Network Resource:"));
    &hostnameoption();
    &action_table_row_end();

    if ($errpos == 102) {
	&errmsg();
    }
    &action_table_row_start(gettext("Resource Name:"));
    if ($FORM{"rs"} eq "") {
	# Find a resource name that is not in use
	for ($i = 1; $i<10; $i++) {
	    $FORM{"rs"} = "nfs-$i";
	    if (! defined $rsdesc{$FORM{"rs"}}) {
		last;
	    }
	    $FORM{"rs"} = "";
	}
    }
    &input("rs");
    &action_table_row_end();
    &action_table_end();

print "<br>\n";
    &submit_table_start();
    &submit("cmd_setup", gettext("Continue"));
    &submit_table_end();
}

# Decide if we want to create or modify - called from cmd_mainpage
sub decide {

    print "<h2>".gettext("Modify an existing resource or create a new resource.\n");
    &action_table_start();
    &action_table_row_start(gettext("Resource to Modify:"));

    print "<select name=\"oldrs\">\n";

    my ($i);
    foreach $i (keys %rsrt) {
	if ($rsrt{$i} eq "SUNW.nfs") {
	    &option("oldrs", $i, $i);
	}
    }
    print "</select>\n";
    &action_table_row_end();
    &action_table_end();

    $FORM{"create"} = 1;
    &hidden("create");

    &submit_table_start();
    &submit("cmd_modify", gettext("Modify"));
    &submit("cmd_mainpage", gettext("Create New"));
    &submit_table_end();
}

# Next part of configuration
sub cmd_setup {
    &wizard_start;
    print $q->_script($q->yoke_jscript("c"));
    &getscrgconfig();

    if (! -d $FORM{"exportdir"}) {
	&errpage(gettext("Directory does not exist"), 2, "cmd_mainpage");
    }

    if (! &isglobal($FORM{"exportdir"})) {
	&errpage(gettext("Directory is not global"), 2, "cmd_mainpage");
    }

    &validatelh("cmd_mainpage");

    my ($lh, $lhrg, $lhrs, $lhmode) = split(/:/, $FORM{"lh"});

    &getrgprops($lhrg);
    # Make sure we aren't clobbering an existing Pathprefix
    if ($rgprops{"Pathprefix"} && $rgprops{"Pathprefix"} ne $FORM{"admindir"}) {
	$FORM{"admindir"} = $rgprops{"Pathprefix"};
	if (! -d $FORM{"admindir"} || ! &isglobal($FORM{"admindir"})) {
	    &errpage($q->sprintfn(gettext("Resource group %1 for %2 has " .
		"Pathprefix set to Invalid value %3.  You must use a " .
		"different logical hostname\n"), 
	    $lhrg, $lh, $rgprops{"Pathprefix"}),
	    101, "cmd_mainpage");
	}
	&errpage($q->sprintfn(gettext("Resource group %1 for %2 has " .
	    "Pathprefix set to %3.  You must either use that prefix or use " .
	    "a different logical hostname\n"),
	    $lhrg, $lh, $rgprops{"Pathprefix"}),
	    3, "cmd_mainpage");
    }

    if (! -d $FORM{"admindir"}) {
	&errpage(gettext("Directory does not exist"), 3, "cmd_mainpage");
    }

    if (! &isglobal($FORM{"admindir"})) {
	&errpage(gettext("Directory is not global"), 3, "cmd_mainpage");
    }

    print "<h2>".gettext("Configuring and starting HA-NFS")."</h2>\n";

    undef @Cluster::common::nodes;
    $Cluster::common::nodes[0] = &Cluster::Common::get_hostname;
    my $admindir = $FORM{"admindir"}."/SUNW.nfs";
    my $exportdir = $FORM{"exportdir"};
    my $resource = $FORM{"rs"};
    my $CMD = "/cgi-bin/ds/nfs-script.pl";
    my (%args) = (admindir => $admindir, exportdir => $exportdir,
	resource => $resource);
    my $cluster = new Cluster::Common;
    my $err = $cluster->run_parallel($CMD, 1, \%args);
    if ($err) {
	    &errmsg(sprintf(gettext("Admin file update script failed %s"), $errmsg));
	return;
    }

    print sprintf("<p>".gettext("Updated admin file %s/%s\n"), $admindir,
	$resource);
    print "<p>" . gettext("Now creating resource group and resource\n");

    my $cmd = "scrgadm -c -g $lhrg -y Pathprefix=".$FORM{"admindir"};
    my $status = &runsuid($cmd);
    if ($status != 0) {
	&errmsg(gettext("Failure"));
	return $status;
    }

    $status = &createservice("-t SUNW.nfs");
    if ($status) {
	return;
    }
    print "<p>".gettext("You can mount this NFS directory: ")."<tt>mount $lh:".$FORM{"exportdir"}." /mnt</tt>\n";
    print "<p>".gettext("Click to modify this resource or add directories:\n");
    $FORM{"oldrs"} = $FORM{"rs"};
    &hidden("oldrs");
    &hidden("create");
    &submit("cmd_modify", gettext("Continue"));
    &reloadmenu();
    print $q->_script($q->yoke_jscript("rg-rs-".$FORM{"rs"}));

}

# Modify an existing NFS resource (in $FORM{"oldrs"}).
sub cmd_modify {
    my $i;
    &wizard_start;
    print $q->_script($q->yoke_jscript("c"));
    &getscrgconfig();
    if ($FORM{"oldrs"} eq "") {
	&errmsg(gettext("No resource selected\n"));
	return;
    }

    # Find the resource group
    my $rs = $FORM{"oldrs"};
    my $rg = $rsrg{$rs};
    $FORM{"rg"} = $rg;
    if ($rg eq "") {
	&errmsg(gettext("Couldn't determine resource group\n"));
	return;
    }

    # Find the Pathprefix

    &getrgprops($rg);

    my $pathprefix = $rgprops{"Pathprefix"};
    if ($pathprefix eq "") {
	&errmsg(gettext("Couldn't determine Pathprefix\n"));
	return;
    }

    my $admin = "$pathprefix/SUNW.nfs/dfstab.$rs";
    if (! -e $admin) {
	&errmsg(sprintf(gettext("NFS admin file %s does not exist\n"), $admin));
	return;
    }

    if (!open(ADMIN, "<$admin")) {
	&errmsg(sprintf(gettext("Couldn't open NFS admin file %s\n"), $admin));
	return;
    }

    print "<h3>".sprintf(gettext("Add or modify HA-NFS exported directories in %s"), $rs)."</h3>\n";
    print "<p>".gettext("You may modify any of the currently exported directories below.\n");
    print "<br>".gettext("You may remove any of the directories from the export file by setting the \"action\" to \"Don't export\".\n");
    print "<br>".gettext("You may add up to 5 additional directories at a time to export.\n");
    print "<br>\n";

    my $parseerror = "";

    if ($errpos == 100) {
	&errmsg();
    }

    my (@headers) = (gettext("Exported Directory"), "25%",
	gettext("Access Options"), "25%",
	gettext("Description"), "25%",
	gettext("Action"), "25%");

    $q->start_prop_table("", \@headers);

    # Process admin file
    my @toks;
    my $tok;
    my $lineno = 0;
    ADMINLINE: while (<ADMIN>) {
	chop;
	my $line = $_;
	if (/^#/ || /^\s*$/) {
	    next ADMINLINE;
	}

	# Break line into tokens
	undef @toks;
	while (1) {
	    if (/^\s*$/) { # Nothing left
		last;
	    } elsif (/^\s*"([^"]*)"\s*(.*)/) { # Quoted stuff
	    } elsif (/^\s*'([^']*)'\s*(.*)/) { # Quoted stuff
	    } elsif (/^\s*(\S+)\s*(.*)/) { # stuff
	    } else { # Shouldn't happen
		print gettext("Parse error\n");
		last;
	    }
	    ($tok, $_) = ($1, $2);
	    push (@toks, $tok);
	}

	if ($toks[0] !~ /share/) {
	    $parseerror = $line;
	    next ADMINLINE;
	}
	shift @toks;

	my $desc = "";
	my $options = "";

	while ($toks[0] =~ /^-/) {
	    if ($toks[0] =~ /^-F/) {
	    } elsif ($toks[0] =~ /-o/) {
		$options = $toks[1];
	    } elsif ($toks[0] =~ /-d/) {
		$desc = $toks[1];
	    } else {
		$parseerror = $line;
		next ADMINLINE;
	    }
	    shift @toks;
	    shift @toks;
	}
	my $path = $toks[0];
	$q->start_prop_tr();
	$q->start_prop_td();
	print "<input name=\"path$lineno\" type=\"text\" value=\"$path\">\n";
	$q->end_prop_td();
	$q->start_prop_td();
	print "<input name=\"options$lineno\" type=\"text\" value=\"$options\">\n";
	$q->end_prop_td();
	$q->start_prop_td();
	print "<input name=\"desc$lineno\" type=\"text\" value=\"$desc\">\n";
	$q->end_prop_td();
	$q->start_prop_td();
	print "<select name=\"action$lineno\">\n";
	print "<option value=\"keep\" selected>".gettext("Export")."</option>\n";
	print "<option value=\"delete\" >".gettext("Don't export")."</option>\n";
	print "</select>\n";
	$q->end_prop_td();
	$q->end_prop_tr();

	$lineno++;
    }
    close(ADMIN);

    # New entries
    for ($i = 0; $i < 5; $i++) {
	$q->start_prop_tr();
	$q->start_prop_td();
	print "<input name=\"path$lineno\" type=\"text\">\n";
	$q->end_prop_td();
	$q->start_prop_td();
	print "<input name=\"options$lineno\" type=\"text\">\n";
	$q->end_prop_td();
	$q->start_prop_td();
	print "<input name=\"desc$lineno\" type=\"text\">\n";
	$q->end_prop_td();
	$q->start_prop_td();
	print gettext("Export")."\n";
	$q->end_prop_td();
	$q->end_prop_tr();

	$lineno++;
    }

    $q->end_prop_table();

    if ($parseerror) {
	&errmsg(sprintf(gettext("Couldn't parse line %s\n"), $parseerror), 0);
    }

    &hidden("oldrs");
    &hidden("rg");
    $FORM{"admin"} = $admin;
    &hidden("admin");
    &submit_table_start();
    &submit("cmd_modify_1", gettext("Continue"));
    &submit_table_end();
}

# Modify an existing NFS resource (in $FORM{"oldrs"}).
# This is the final step
sub cmd_modify_1 {
    &wizard_start;
    print $q->_script($q->yoke_jscript("c"));
    &getscrgconfig();

    if ($FORM{"oldrs"} eq "" || $FORM{"rg"} eq "" || $FORM{"admin"} eq "") {
	&errmsg(gettext("Resource / group / admin info missing\n"));
	return;
    }

    my $newadmin = "/var/cluster/spm/nfs.$$";
    if (! open(ADMIN, "| /bin/pfexec /bin/tee $newadmin > /dev/null")) {
	&errmsg(sprintf(gettext("Couldn't open %s\n"), $newadmin));
	return;
    }

    # Process the information from the table to generate new file
    my $lineno;
    for ($lineno = 0; $FORM{"path$lineno"} || $FORM{"action$lineno"}; $lineno++) {
	my $path = $FORM{"path$lineno"};
	my $options = $FORM{"options$lineno"};
	my $desc = $FORM{"desc$lineno"};
	my $action = $FORM{"action$lineno"};
	if (! &isglobal($path)) {
	    &errmsg(sprintf(gettext("%s is not global: skipping\n"), $path));
	    next;
	}
	if ($action eq "delete" || $path eq "") {
	    # Skip
	    next;
	} else {
	    if ($options eq "") {
		$options = "rw";
	    }
	    if ($desc) {
		$desc = " -d \"$desc\"";
	    }
	    print ADMIN "share -F nfs -o $options$desc $path\n";
	}
    }

    if (!close(ADMIN)) {
	&errmsg(sprintf(gettext("Couldn't open %s\n"), $newadmin));
	return;
    }

    print "<p>".gettext("Disabling resource\n");
    my $cmd = "scswitch -n -j ".$FORM{"oldrs"};
    my $status = &runsuid($cmd);
    if ($status != 0) {
	&errmsg(gettext("Failure"));
	return $status;
    }

    my $failed = 0;

    print "<p>".sprintf(gettext("Updating admin file %s"), $FORM{"admin"})."\n";
    my $CMD = "/bin/mv $newadmin ".$FORM{"admin"};
    if (system("/bin/pfexec $CMD")) {
	&errmsg(gettext("Failure"));
	return "";
    }

    print "<h3>".gettext("Enabling resource")."</h3>\n";
    $cmd = "scswitch -e -j ".$FORM{"oldrs"};
    $status = &runsuid($cmd);
    if ($status != 0) {
	&errmsg(gettext("Failure"));
	return $status;
    }

    if (! $failed) {
	&okmsg(gettext("HA-NFS restarted with new configuration file\n"));
    }
}
