#!/usr/perl5/bin/perl -w
#
# Copyright (c) 2001 by Sun Microsystems, Inc.
# All rights reserved.
#
#ident	"@(#)patchadd_local.pl	1.12	01/11/28 SMI"
#
# Add patches locally, given the correct path.  Please note that the patches
# have already been uncompressed and are ready to be applied.

require 5.005;
use strict;
use lib '/opt/SUNWscvw/lib/perl';
use Cluster::Cgi;
use Cluster::Common;

# Create the html query object
my $q = new Cluster::Cgi;
my $cluster = new Cluster::Common;

# Print the header
print $q->header();

# Fork so that we can exit to the parent immediately
if (fork()) {
    exit;
}

# Get the arguments
my $uncompressed_patch_path = $q->param("uncompressed_patch_path");
my $patchadd_error_log = $q->param("patchadd_error_log");

# We must use the pfexec command if root permission is needed to successfully
# execute a certain command.  (e.g. patchadd, umount, pgkrm, etc )  When using
# pfexec, please be sure to include the absolute path to the command or pfexec
# will fail.
my $PFEXEC = "/usr/bin/pfexec";

# Set the error sentinel
my @errors;

# Remove the BASEDIR environment variables
delete @ENV{'BASEDIR', 'CLIENT_BASEDIR'};

# Get the list of patches to add
my @patchadd = `/opt/SUNWscvw/cgi-bin/installation/order_patches.ksh $uncompressed_patch_path`;

# Add the patches!
foreach my $patch (@patchadd) {
    chomp $patch;

    # Skip if $patch is NOT a directory
    next if ( ! -d "$uncompressed_patch_path/$patch" );
    # Skip if $patch is not a valid patch directory
    next if ( ! -e "$uncompressed_patch_path/$patch/.diPatch" &&
        ! -e "$uncompressed_patch_path/installpatch" );

    # Add the patch
    my $result = `$PFEXEC /usr/sbin/patchadd -M $uncompressed_patch_path $patch`;
    if ($? != 0) {
        chomp($result);
        $result =~ tr/\n/ /; # Remove all new lines
        $cluster->syslog("patchadd of $uncompressed_patch_path/$patch FAILED ".
		"with error ($result).");
        push (@errors, "patchadd of $patch failed with error <br><br>$result");
    } else {
        $cluster->syslog("patchadd of $uncompressed_patch_path/$patch SUCCEEDED");
    }
}

# Touch the file that indicates we're done
`touch /var/cluster/spm/done_patches`;
if ($? != 0) {
    $cluster->syslog("touch /var/cluster/spm/done_patches FAILED!");
    push (@errors, "touch /var/cluster/spm/done_patches FAILED!");
}

# If there were any errors in this script, then print them to the touched file.
if ( @errors) {
    # If there is already an error log, then save it under a different name.
    if ( -f "$patchadd_error_log" ) {
         `mv $patchadd_error_log $patchadd_error_log.$$`;
          if ($? != 0) {
              $cluster->syslog("mv $patchadd_error_log FAILED!");
          } else {
              $cluster->syslog("Successful rename of $patchadd_error_log to $patchadd_error_log.$$");
          }
    }
    open(FILE, ">$patchadd_error_log");
    foreach my $e (@errors) {
        print FILE "$e\n";
    }
    close(FILE);
}

# Exit completely
CORE::exit(0);
