#!/usr/perl5/bin/perl -w
#
# Copyright (c) 2001 by Sun Microsystems, Inc.
# All rights reserved.
#
#ident	"@(#)verify_paths.pl	1.11	01/07/31 SMI"
#
# Make sure that the CDROM and patch paths are reachable.

require 5.005;
use strict;
use lib '/opt/SUNWscvw/lib/perl';
use Cluster::Cgi;
use Cluster::Install;
use Sun::Solaris::Utils qw(gettext);

# Create a cgi object
my $q = new Cluster::Cgi;

# Create a install object
my $install = new Cluster::Install;

# Print the header
print $q->header();

# Get the paths
my $cdrom_path = $q->param("cdrom_path");
my $solaris_cdrom_path = $q->param("solaris_cdrom_path");
my $ds_cdrom_path = $q->param("ds_cdrom_path");
my $patch_path = $q->param("patch_path");

# Get the hostname
my $hostname = $install->get_hostname();

# Create list of errors
my (@errors);

# Verify that scinstall exists under the given cdrom path
my $scinstall_path = $install->get_scinstall_path($cdrom_path);
if (! -T $scinstall_path) {
        push @errors, $q->sprintfn(gettext("%1 could not access scinstall " .
					   "under %2"), $hostname, $cdrom_path);
}

# Verify the patch directory path
if (defined $patch_path && ! -d $patch_path) {
    push @errors, $q->sprintfn(gettext("%1 could not access %2"),
	$hostname, $patch_path);
}

# Verify the Sun Cluster Data Services CDROM path
if (defined $ds_cdrom_path && ! -T "$ds_cdrom_path/.cdtoc") {
    push @errors, $q->sprintfn(gettext("%1 could not access %2"),
	$hostname, $ds_cdrom_path);
}

# Verify the Solaris CDROM path
if (defined $solaris_cdrom_path && ! -T "$solaris_cdrom_path/.cdtoc") {
    push @errors, $q->sprintfn(gettext("%1 could not access %2"),
	$hostname, $solaris_cdrom_path);
}

# If we found errors, print them and exit gracefully
if ($#errors > -1) {
    foreach my $error (@errors) {
	print "$error\n";
    }

    exit(0);
}
