#
# Copyright (c) 2001 by Sun Microsystems, Inc.
# All rights reserved.
#
#ident	"@(#)Menu.pm	1.27	01/10/16 SMI"
#
# Menu class

package Cluster::Menu;
use strict;
use Cluster::Rg;
use Cluster::Common;
use Cluster::Device;
use Cluster::Quorum;
use Cluster::Network;
use Cluster::Transport;
use Sun::Solaris::Utils qw(gettext);
use vars qw(@ISA $VERSION);
$VERSION = '1.00';
@ISA = qw(Cluster::Common);

##############################################################################
#
# Class Variables
#
##############################################################################

my $rg = new Cluster::Rg;
my $device = new Cluster::Device;
my $quorum = new Cluster::Quorum;
my $network = new Cluster::Network;
my $transport = new Cluster::Transport;

# Icons in the menu

my $CLUSTERIMG = "/images/jmenu/sun-cluster_16_pad.gif";
my $NODEIMG = "/images/jmenu/node_16_pad.gif";
my $LOGIMG = "/images/jmenu/log_16_pad.gif";
my $PROCESSIMG = "/images/jmenu/process_16_pad.gif";
my $NAFOIMG = "/images/jmenu/nafo_16_pad.gif";
my $RGIMG = "/images/jmenu/rsrcgrps_16_pad.gif";
my $RSIMG = "/images/jmenu/rsrc_16_pad.gif";
my $QUORUMIMG = "/images/jmenu/quorum_16_pad.gif";
my $TRANSPORTIMG = "/images/jmenu/adapter_16_pad.gif";
my $NODEADAPTERIMG = "/images/jmenu/node_adapt_16_pad.gif";
my $ADAPTERIMG = "/images/jmenu/adapter_16_pad.gif";
my $JUNCTIONIMG = "/images/jmenu/junction_16_pad.gif";
my $DEVICEIMG = "/images/jmenu/devicegrps_16_pad.gif";
my $FOLDERIMG = "/images/jmenu/folder_16_pad.gif";

#Pages are yoked to the tree using the following keys:

#Top-level:	"c"
# Nodes:	"n"
#  Node x:	"n-x"
#   NAFO:	"nf-x"
#   Processes:	"np-x"
#   Logs:	"nl-x"
# Res Group:	"rg"
#  R Group x:	"rg-x"
#   Res x:	"rg-rs-x"
# Transports:	"t"
#  Nodes/Adap:	"ta"
#   Node x:	"ta-x"
#  Junctions:	"tj"
#   Hub x:	"tj-x"
# Glob Devices:	"d"
#  Device x:	"d-x"
# Quorum:	"q"
#  Device x:	"q-x"

##############################################################################
#
# Class Methods
#
##############################################################################

sub new {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $self  = {};
    bless ($self, $class);
    return $self;
}

##############################################################################
#
# Folder Tree Functions
#
##############################################################################

sub folder_tree_start($$$$$) {
	my ($self, $q, $mode, $name, $link, $icon, $key) = @_;

	if ($mode eq "basic") {
		print $q->start_p({ CLASS => 'tree-level0' });
		print $q->start_a({ HREF => $link, TARGET => "content" });
		print sprintf(gettext("%s Cluster"), $name);
		print $q->end_a();
		print $q->end_p();
	} else {
		print "\nfoldersTree = gFld('";
		print $q->start_span({ CLASS => 'top-tree-node-text' });
		print sprintf(gettext("%s Cluster"), $name);
		print $q->end_span();
		print "', '$link', ";	
		print "'$name', ";	
		print "'$icon', ";	
		print "'$key')\n";	
	}
}

sub folder_level_one($$$$) {
	my ($self, $q, $mode, $name, $link, $icon, $key) = @_;

	if ($mode eq "basic") {
		print $q->start_p({ CLASS => 'tree-level1' });
		print $q->start_a({ HREF => $link, TARGET => "content" });
		print $name;
		print $q->end_a();
		print $q->end_p();
	} else {
		print "  aux1 = insFld(foldersTree, gFld('";
		print $q->start_span({ CLASS => 'tree-node-text' });	
		print $name;
		print $q->end_span();
		print "', '$link', ";	
		print "'$name', ";	
		print "'$icon', ";	
		print "'$key'))\n";	
	}
}

sub folder_level_two($$$$) {
	my ($self, $q, $mode, $name, $link, $icon, $key) = @_;

	if ($mode eq "basic") {
		print $q->start_p({ CLASS => 'tree-level2' });
		print $q->start_a({ HREF => $link, TARGET => "content" });
		print "&nbsp;*&nbsp;";
		print $name;
		print $q->end_a();
		print $q->end_p();
	} else {
		print "    aux2 = insFld(aux1, gFld('";
		print $q->start_span({ CLASS => 'tree-node-text' });	
		print $name;
		print $q->end_span();
		print "', '$link', ";	
		print "'$name', ";	
		print "'$icon', ";	
		print "'$key'))\n";	
	}
}

sub folder_level_three($$$$) {
	my ($self, $q, $mode, $name, $link, $icon, $key) = @_;

	if ($mode eq "basic") {
		print $q->start_p({ CLASS => 'tree-level3' });
		print $q->start_a({ HREF => $link, TARGET => "content" });
		print "&nbsp;&nbsp;+&nbsp;";
		print $name;
		print $q->end_a();
		print $q->end_p();
	} else {
		print "      aux3 = insFld(aux2, gFld('";
		print $q->start_span({ CLASS => 'tree-node-text' });	
		print $name;
		print $q->end_span();
		print "', '$link', ";	
		print "'$name', ";	
		print "'$icon', ";	
		print "'$key'))\n";	
	}
}

sub leaf_level_one($$$$) {
	my ($self, $q, $mode, $name, $link, $icon, $key) = @_;

	if ($mode eq "basic") {
		print $q->start_p({ CLASS => 'tree-level1' });
		print $q->start_a({ HREF => $link, TARGET => "content" });
		print $name;
		print $q->end_a();
		print $q->end_p();
	} else {
		print "    insDoc(foldersTree, gLnk(0, '";
		print $q->start_span({ CLASS => 'tree-node-text' });	
		print $name;
		print $q->end_span();
		print "', '$link', ";	
		print "'$name', ";	
		print "'$icon', ";	
		print "'$key'))\n";	
	}
}

sub leaf_level_two($$$$) {
	my ($self, $q, $mode, $name, $link, $icon, $key) = @_;

	if ($mode eq "basic") {
		print $q->start_p({ CLASS => 'tree-level2' });
		print $q->start_a({ HREF => $link, TARGET => "content" });
		print "&nbsp;+&nbsp;";
		print $name;
		print $q->end_a();
		print $q->end_p();
	} else {
		print "    insDoc(aux1, gLnk(0, '";
		print $q->start_span({ CLASS => 'tree-node-text' });	
		print $name;
		print $q->end_span();
		print "', '$link', ";
		print "'$name', ";	
		print "'$icon', ";	
		print "'$key'))\n";	
	}
}

sub leaf_level_three($$$$) {
	my ($self, $q, $mode, $name, $link, $icon, $key) = @_;

	if ($mode eq "basic") {
		print $q->start_p({ CLASS => 'tree-level3' });
		print $q->start_a({ HREF => $link, TARGET => "content" });
		print "&nbsp;&nbsp;+&nbsp;";
		print $name;
		print $q->end_a();
		print $q->end_p();
	} else {
		print "      insDoc(aux2, gLnk(0, '";
		print $q->start_span({ CLASS => 'tree-node-text' });	
		print $name;
		print $q->end_span();
		print "', '$link', ";
		print "'$name', ";	
		print "'$icon', ";	
		print "'$key'))\n";	
	}
}

sub reload_menu ($$) {
	my ($self, $q) = @_;
	print $q->start_script();
	print "top.menu.location.reload(true)";
	print $q->end_script();
}

##############################################################################
#
# Menu data gathering functions
#
##############################################################################

#
# Get the cluster name
#

sub getClusterName ($$$$) {
	my ($self, $q, $mode, $clustername) = @_;
	$self->folder_tree_start($q, $mode, $clustername,
				 "/cgi-bin/cluster/cluster_status.pl",
				 $CLUSTERIMG, "c");
}

#
# Print nodes menu
#

sub getNodes ($$$) {
	my ($self, $q, $mode) = @_;
	my (@NODELIST, $NODE, $i, $j, $title);

	$title = gettext("Nodes");
	$self->folder_level_one($q, $mode, $title,
				"/cgi-bin/node/node_status.pl",
				$FOLDERIMG, "n");

	@NODELIST = $rg->get_nodelist();

	for ($i=0; $i<=$#NODELIST; $i++) {

		$NODE = $NODELIST[$i];
		$self->folder_level_two($q, $mode, $NODE,
			"/cgi-bin/node/node_info.pl?node=$NODE",
			$NODEIMG, "n-$NODE");

		$title = gettext("NAFO Groups");
		$self->leaf_level_three($q, $mode, $title,
				"/cgi-bin/network/network_view.pl?node=$NODE",
				$NAFOIMG, "nf-$NODE");

		$title = gettext("Processes");
		$self->leaf_level_three($q, $mode, $title,
			"/cgi-bin/process/process.pl?node=$NODE",
			$PROCESSIMG, "np-$NODE");

		$title = gettext("System Logs");
		$self->leaf_level_three($q, $mode, $title,
			"/cgi-bin/log/log.pl?node=$NODE",
			$LOGIMG, "nl-$NODE");
	}
}

#
# Print resource groups menu
#

sub getResourceGroups ($$$) {
	my ($self, $q, $mode) = @_;	
	my ($i, $j, @RG_LIST, $RG, @RS_LIST, $RS, $title);

	$title = gettext("Resource Groups");
	@RG_LIST = $rg->get_rglist();

	# If we don't have any resource groups, make the rg link a leaf
	if ($#RG_LIST == -1) {
		$self->leaf_level_one($q, $mode, $title,
				      "/cgi-bin/rg/rg_view.pl",
				      $RGIMG, "rg");
	} else {
		$self->folder_level_one($q, $mode, $title,
					"/cgi-bin/rg/rg_view.pl",
					$FOLDERIMG, "rg");

		for ($i=0; $i<=$#RG_LIST; $i++) {
			$RG = $RG_LIST[$i];		
			@RS_LIST = $rg->get_rslist($RG);

			# If we don't have any resources make the link a leaf
			if ($#RS_LIST == -1) {
				$self->leaf_level_two($q, $mode, $RG,
				    "/cgi-bin/rg/rg_view.pl?rg=$RG",
				    $RGIMG, "rg-$RG");
			} else {
				$self->folder_level_two($q, $mode, $RG,
				    "/cgi-bin/rg/rg_view.pl?rg=$RG",
				    $RGIMG, "rg-$RG");

				for ($j=0; $j<=$#RS_LIST; $j++){
					$RS = $RS_LIST[$j];
					$self->leaf_level_three($q, $mode, $RS,
					     "/cgi-bin/rg/rs_view.pl?" .
					     "rs=$RS&rg=$RG",
					     $RSIMG, "rg-rs-$RS");
				}
			}
		}
	}
}

#
# Print transport menu
#

sub getTransports ($$$) {
	my ($self, $q, $mode) = @_;
	my (@NODELIST, $NODE, $i, @JCTLIST, $JCT, $title);

	$title = gettext("Transports");
	$self->folder_level_one($q, $mode, $title,
				"/cgi-bin/transport/transport_view.pl",
				$FOLDERIMG, "t");

	$title = gettext("Nodes/Adapters");
	$self->folder_level_two($q, $mode, $title,
			"/cgi-bin/transport/transport_view.pl?type=Nodes",
			$FOLDERIMG, "ta");

	@NODELIST = $transport->get_nodelist();
	for ($i=0; $i<=$#NODELIST; $i++) {
		$NODE = $NODELIST[$i];
		$self->leaf_level_three($q, $mode, $NODE,
		     "/cgi-bin/transport/transport_view.pl" .
					"?type=Node&name=$NODE",
		    $NODEADAPTERIMG, "ta-$NODE");

	}

	$title = gettext("Junctions");
	$self->folder_level_two($q, $mode, $title,
			"/cgi-bin/transport/transport_view.pl?type=Junctions",
			$FOLDERIMG, "tj");

	$transport->get_commandoutput();
	@JCTLIST = $transport->get_junctionlist();

	for ($i=0; $i<=$#JCTLIST; $i++) {
		$JCT = $JCTLIST[$i]{'name'};
		$self->leaf_level_three($q, $mode, $JCT,
					"/cgi-bin/transport/transport_view.pl".
					"?type=Junction&name=$JCT",
					$JUNCTIONIMG, "tj-$JCT");
	}
}

#
# Print global device menu
#

sub getGlobalDevices ($$$) {
	my ($self, $q, $mode) = @_;
	my (@GDEVLIST, $GDEV, $i, $title);

	$title = gettext("Global Devices");

	$device->get_commandoutput();
	@GDEVLIST = $device->get_devicegrouplist();

	if ($#GDEVLIST == -1) {
		$self->leaf_level_one($q, $mode, $title,
				"/cgi-bin/device/device_view.pl",
				$DEVICEIMG, "d");
	} else {
		$self->folder_level_one($q, $mode, $title,
				"/cgi-bin/device/device_view.pl",
				$FOLDERIMG, "d");
		for ($i=0; $i<=$#GDEVLIST; $i++) {
			$GDEV = $GDEVLIST[$i]{'name'};
			$self->leaf_level_two($q, $mode, $GDEV,
			   "/cgi-bin/device/device_view.pl?devicegroup=$GDEV",
			    $DEVICEIMG, "d-$GDEV");
		}
	}
}

#
# Print quorum menu
#

sub getQuorum ($$$) {
	my ($self, $q, $mode) = @_;
	my ($myJS, @QDEVLIST, $QDEV, $i, $title);

	$title = gettext("Quorum");

	$quorum->get_scconfpout();
	@QDEVLIST = $quorum->get_quorumdevicelist();

	if ($#QDEVLIST == -1) {
		$self->leaf_level_one($q, $mode, $title,
				      "/cgi-bin/quorum/quorum_view.pl",
				      $QUORUMIMG, "q");
	} else {
		$self->folder_level_one($q, $mode, $title,
					"/cgi-bin/quorum/quorum_view.pl",
				      $FOLDERIMG, "q");
		for ($i=0; $i<=$#QDEVLIST; $i++) {
			$QDEV = $QDEVLIST[$i]{'name'};
			$self->leaf_level_two($q, $mode, $QDEV,
		          "/cgi-bin/quorum/quorum_view.pl?quorumdevice=$QDEV",
			  $QUORUMIMG, "q-$QDEV");
		}
	}
}

# Return true
1;
